<?php
include_once ('soap/MoodleSoapClasses.php');

function soap_encode($str) {
  return $str;
  // return recode('iso-8859-1..utf-8', $str);
}

class MoodleSoapCallback {
  function MoodleSoapCallback() {
  } // MoodleSoapCallback()

  function condition($input) {
    $segs = split(',', $input);
    if (count($segs) > 1) {
      for($i = 0; $i < count($segs); ++$i)
        $segs[$i] = $segs[$i] * 1;
      return 'IN ('.implode(',', $segs).')';
    }
    else {
      return '= '.($segs[0] * 1);
    }
  } // condition ()
  
  /**
   * moodle.MoodleSoapCallback::getCourses 
   * SOAP operation: getCourses
   *
   * Liste von Moodle-Kursen
   *
   * @param conditions     ArrayOfCondition (class MoodleSoapArrayOfCondition)
   *
   * @return courses   ArrayOfCourse (class MoodleSoapArrayOfCourse)
   *
   */
  function getCourses(&$conditions) {
    global $CFG;
    include_once('lib/moodlelib.php');
    include_once('lib/datalib.php');

    $validExp = array('category'   => '',
                      'course'     => 'id',
                      'title'      => 'fullname',
                      'shortcut'   => 'shortname',
                      'summary'    => 'summary',
                      'allowGuest' => 'guest',
                      'public'     => 'password',
                      'teacher'    => '');

    $validOps = array('=', 'like');

    
    $match         = array();
    $categoryId    = 'all';
    $teacherId     = null;
    $courseId      = null;
    $error         = null;
    
    do {
      if (is_array($conditions)) {
        foreach($conditions AS $condition) {
          if (!in_array($condition->op, $validOps)) {
            $error = new SOAP_Fault('Invalid condition: "'.$condition->expr.$condition->op.$condition->value.'"');
          }
          switch ($condition->expr) {
            case 'category':
              $categoryId = $condition->value;
              break;
            case 'title':
            case 'shortcut':
            case 'summary':
              if ($condition->op == '=') {
                $match[] = '(stripos($course->'.$validExp[$condition->expr].', \''.$condition->value.'\') === false);';
              }
              else {
                $match[] = '(eregi(\''.$condition->value.'\', $course->'.$validExp[$condition->expr].') == false);';
              }
              break;
            case 'allowGuest':
                $condition->value = ($condition->value == 'false' || !$condition->value ? 'false' : 'true');
                $match[] = '('.$condition->value.' != $course->'.$validExp[$condition->expr].');';
                break;
            case 'public':
              $condition->value = ($condition->value == 'false' || !$condition->value ? 'false' : 'true');
              $match[] = '('.$condition->value.' != !($course->password));';
              break;
            case 'teacher':
              $teacherId = $condition->value;
              break;
            case 'course':
              $courseId  = $condition->value;
              break;
              
            default:
              $error = new SOAP_Fault('Invalid condition: "'.$condition->expr.$condition->op.$condition->value.'"');
          }
          if (PEAR::isError($error)) {
            break;
          }
        }
      }
      if (PEAR::isError($error)) {
        $result = $error;
        break;
      }

      $courseStmt   = '';
      $categoryStmt = '';
      $teacherStmt  = '';

      if ($courseId) {
        $courseStmt = ' AND '.$CFG->prefix.'course.id '.$this->condition($courseId);
      }
      if ($teacherId) {
        $myCourses = get_my_courses ($teacherId);

        $courses   = array ();

        foreach (array_keys ($myCourses) as $courseid) {
          $context = get_context_instance(CONTEXT_COURSE, $courseid);

          if ( has_capability('moodle/course:update', $context, $teacherId, false)) 
            $courses[] = $courseid;
        }

        if (count ($courses) > 0) {
          $teacherStmt = ' AND '.$CFG->prefix.'course.id IN ('.implode (',', $courses).')';
        }
        else {
          $teacherStmt = ' AND 1 = 2';
        }
      }
      if ($categoryId) {
        if ($categoryId != 'all')
          $categoryStmt = ' AND '.$CFG->prefix.'course.category '.$this->condition($categoryId);
      }

      $q = '
SELECT 
  '.$CFG->prefix.'course.*, 
  '.$CFG->prefix.'course_categories.name AS ctitle, 
  '.$CFG->prefix.'course_categories.id AS cid 
FROM 
  '.$CFG->prefix.'course,
  '.$CFG->prefix.'course_categories 
WHERE 
  '.$CFG->prefix.'course.category = '.$CFG->prefix.'course_categories.id 
  AND '.$CFG->prefix.'course.visible = 1 '
  .$courseStmt
  .$categoryStmt
  .$teacherStmt.'
ORDER BY
  '.$CFG->prefix.'course_categories.id,
  '.$CFG->prefix.'course.fullname ASC';

      $courses = get_records_sql($q);  

      $resCourses = array();
      $count = 0;

      foreach($courses AS $id => $course) {
        do {
          if ($course->visible) {
            $skip = false;
            foreach($match AS $condition) {
              eval('$skip = '.$condition);
              if ($skip) {
                break;
              }
            }
            if (!$skip) {
              ++$count;
              $teachersStr = '';
              $context = get_context_instance(CONTEXT_COURSE, $id);
              if ($teachers = get_users_by_capability($context, 'moodle/course:update', 'u.*,ra.hidden', 'r.sortorder ASC', '', '', '', '', false)) {
                  foreach ($teachers as $teacher) {
                    if (!$teacher->hidden) {
                      if ($teachersStr) 
                        $teachersStr .= ', ';
                      $teachersStr .= $teacher->firstname.' '.$teacher->lastname;
                    }
                }
              }

              $resCourses[] = new MoodleSoapCourse($id,
                                                   soap_encode($course->fullname),
                                                   soap_encode($course->shortname),
                                                   soap_encode($course->summary),
                                                   !($course->password),
                                                   $course->guest,
                                                   'http://lms.hu-berlin.de/moodle/course/view.php?id='.$id,
                                                   soap_encode($teachersStr),
                                                   $course->cid,
                                                   soap_encode($course->ctitle));
            }
          }
        } while(false);
        if ($count >= 100) break;
      }
      $result = new MoodleSoapArrayOfCourse($resCourses);
    } while (false);
    return $result;
  } // getCourses()
  
  /**
   * moodle.MoodleSoapCallback::getCategories 
   * SOAP operation: getCategories
   *
   * Liste von Moodle Kategorien (HU Struktur hierarchisch)
   *
   * @param conditions     ArrayOfCondition (class MoodleSoapArrayOfCondition)
   *
   * @return categories   ArrayOfCategory (class MoodleSoapArrayOfCategory)
   *
   */
  function getCategories(&$conditions) {
    include_once('lib/moodlelib.php');
    include_once('lib/datalib.php');

    $validExp = array('parentCategory', 'category', 'id', 'parentId', 'title');
    $validOps = array('=', 'like');
    $parentCategory = 'none';
    $categoryId       = null;
    $searchPattern  = '';
    $matchMethod    = null;
    do {
      foreach($conditions AS $condition) {
        if (in_array($condition->expr, $validExp) && in_array($condition->op, $validOps)) {
          if ($condition->expr == 'parentCategory' || $condition->expr == 'parentId') {
            $parentCategory = ($condition->value * 1);
          }
          elseif ($condition->expr == 'title') {
            $searchPattern = $condition->value;
            $matchMethod   = $condition->op;
          }
          elseif ($condition->expr == 'category' || $condition->expr == 'id') {
            $categoryId = $condition->value;
          }
        }
        else {
          $result = new SOAP_Fault('Invalid condition: "'.$condition->expr.$condition->op.$condition->value.'"');
          break;
        }
      }
      if (PEAR::isError($result)) {
        break;
      }
      $categories    = get_categories($parentCategory);
      $resCategories = array();

      foreach($categories AS $id => $category) {
        do {
          if ($category->visible) {
            if ($categoryId && $id != $categoryId) break;
            if ($searchPattern && ($matchMethod == '=' ? stripos($category->name, $title) === false : eregi($searchPattern, $category->name) == false)) break;
            $resCategories[] = new MoodleSoapCategory($id, 
                                                      soap_encode($category->name),
                                                      'http://lms.hu-berlin.de/modev/course/category.php?id='.$id);
          }
        } while(false);
      }
      $result = new MoodleSoapArrayOfCategory($resCategories);
    } while(false);

    return $result;
  } // getCategories()

} // class MoodleSoapCallback
?>
