<!--
/******************************************************************************/
var Moodle = {};


/******************************************************************************/
/**
  * Moodle.Client
  * Proxy Klasse fuer Moodle Soap Aufrufe
  *
  * Beispiel:
  * 
  * // Aufruf der Soap Operation 'foo', als Callback-Objekt
  * // wird ein Objekt der Klasse MyCallback uebergeben. 
  * // Diese Klasse muss eine Methode 'foo' mit folgender 
  * // Signatur implementieren:
  * //   foo(WS.Call call, SOAP.Envelope envelope, String xmlResponse)
  * // 
  * (new Moodle.Client(new MyCallback())).foo(p1, p1);
  *
  */
Moodle.Client = Class.create();
Moodle.Client.prototype = {
  initialize :
    function(callback, endpoint) {
      if (callback) {
        this.callback = callback;
      }
      else {
        this.callback = new Moodle.Callback();
      }
      if (endpoint) {
        this.endpoint = endpoint;
      }
      else {
        this.endpoint = 'http://lms2.cms.hu-berlin.de/ws/soap/?Moodle';
      }
    },
  deserializeResult : 
    function(wsCall, responseEnvelope, responseText) {
      
    },
  

  /**
    * Liste von Moodle-Kursen
    */
  getCourses :
    function(
      /* moodlexsd:ArrayOfCondition[] */
      conditions) {
      var call   = new WS.Call(this.endpoint);
      var method = new WS.QName('getCourses', 'http://lms.hu-berlin.de/moodle');
      call.invoke_rpc(method, new Array(
                                        {name:'conditions', value:conditions}
                                        ),
                      null, // keine Angaben zu encoding ... rpc/literal ist wohl default...?
                      this.callback.getCourses);
    },
  

  /**
    * Liste von Moodle Kategorien (HU Struktur hierarchisch)
    */
  getCategories :
    function(
      /* moodlexsd:ArrayOfCondition[] */
      conditions) {
      var call   = new WS.Call(this.endpoint);
      var method = new WS.QName('getCategories', 'http://lms.hu-berlin.de/moodle');
      call.invoke_rpc(method, new Array(
                                        {name:'conditions', value:conditions}
                                        ),
                      null, // keine Angaben zu encoding ... rpc/literal ist wohl default...?
                      this.callback.getCategories);
    }
  
}



/******************************************************************************/
/**
  * Moodle.Callback
  *
  * Leere Implementierungen aller fuer den Moodle
  * benoetigten Callback Funktionen
  * 
  * VIELE TODOs!
  *  - Trennung generierter Code / eigene Implementierung
  *  - Auswertung des Response
  *  - ...
  */
Moodle.Callback = Class.create();
Moodle.Callback.prototype = {
  initialize :
    function() {
    },
  
  getCourses :
    /**
      * Liste von Moodle-Kursen
      */
    function(wsCall, responseEnvelope, responseText) {
     // erstmal, damit ueberhaupt etwas zu sehen ist:
     alert(responseText);
    },
  
  getCategories :
    /**
      * Liste von Moodle Kategorien (HU Struktur hierarchisch)
      */
    function(wsCall, responseEnvelope, responseText) {
     // erstmal, damit ueberhaupt etwas zu sehen ist:
     alert(responseText);
    }
  
}

/******************************************************************************/
/**
  * Absoluter Alpha Code: 
  * Klassen fuer Soap-Sequences, braucht's, damit die 
  * Serialisierung halbwegs generisch funktionieren kann
  */
SOAP.Sequence = Class.create();
SOAP.Sequence.prototype = {
  initialize :
    function() {
    },
  getValue :
    function(key) {
      return this[key];
    },
  setValue :
    function(key, value) {
      this[key] = value;
    }
}


/******************************************************************************/
Moodle.Condition = Class.create();
Moodle.Condition.prototype = (new SOAP.Sequence()).extend({
  initialize :
    function(/* string */ expr,
             /* string */ op,
             /* string */ value
             ) {
      this.attrs = new Array();
      
      this['expr'] = expr;
      this.attrs.push('expr');
      this['op'] = op;
      this.attrs.push('op');
      this['value'] = value;
      this.attrs.push('value');

    }
});

/******************************************************************************/
Moodle.Course = Class.create();
Moodle.Course.prototype = (new SOAP.Sequence()).extend({
  initialize :
    function(/* int */ id,
             /* string */ title,
             /* string */ shortcut,
             /* string */ summary,
             /* boolean */ public,
             /* boolean */ allowGuest,
             /* string */ location
             ) {
      this.attrs = new Array();
      
      this['id'] = id;
      this.attrs.push('id');
      this['title'] = title;
      this.attrs.push('title');
      this['shortcut'] = shortcut;
      this.attrs.push('shortcut');
      this['summary'] = summary;
      this.attrs.push('summary');
      this['public'] = public;
      this.attrs.push('public');
      this['allowGuest'] = allowGuest;
      this.attrs.push('allowGuest');
      this['location'] = location;
      this.attrs.push('location');

    }
});

/******************************************************************************/
Moodle.Category = Class.create();
Moodle.Category.prototype = (new SOAP.Sequence()).extend({
  initialize :
    function(/* int */ id,
             /* string */ title,
             /* string */ location
             ) {
      this.attrs = new Array();
      
      this['id'] = id;
      this.attrs.push('id');
      this['title'] = title;
      this.attrs.push('title');
      this['location'] = location;
      this.attrs.push('location');

    }
});


-->
