<?php
/* class SoapServer5 
 *
 * Copyright (C) 2005 Christoph Soergel (csoergel@gmx.net)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * For more information, comments or bug reports: Please refer to
 * http://pr17.net or write to csoergel@gmx.net
 */
include_once('SOAP/Server.php');
include_once('soap/soaputils.php');

/**
  * global callback function
  */
function soapRequestCallback($methodName, &$args) {
  $cb =& SoapServer5::callback($v = null);
  return $cb->$methodName($args);
} // soapRequestCallback()

class SoapServer5 extends SOAP_Server {
  
  function & callback(&$callbackObj) {
    static $cb;
    if (is_object($callbackObj)) {
      $cb = $callbackObj;
    }
    return $cb;
  } // callback()
 
  function SoapServer5(&$callbackObj, $namespace) {
    define('SOAP_PROCESSING_START', microtime());
    $this->SOAP_Server();
    $this->addObjectMap($callbackObj, $namespace);
    $this->setCallHandler('soapRequestCallback');
    $this->service($GLOBALS['HTTP_RAW_POST_DATA']);
    flush ();
    die ();
  } // SoapServer5()


  /**
    * overrides SOAP_Server::buildResult()
    *
    * allow multiple return values (very rarely needed)
    */
  function buildResult(&$method_response, &$return_type, $return_name='return', $namespace = '') {
    if (is_array($method_response)) {
      $keys = array_keys($method_response);
      if (count($keys) > 0 && !is_numeric($keys[0])) {
        return $method_response;
      }
    }
    return parent::buildResult($method_response, $return_type, $return_name, $namespace);
  } // buildResult()

  /**
    * overrides SOAP_Server::_sendResponse()
    * 
    * check for SOAP Faults caught by callback object
    */
  function _sendResponse($response = '') {
    $cb =& SoapServer5::callback($v = null);
    if ($cb->fault != null) {
      $this->fault = $cb->fault;
    }
    /* Make distinction between the different SAPIs, running PHP as CGI or
     * as a module. */
    if (stristr(php_sapi_name(), 'cgi') === 0) {
        $hdrs_type = 'Status:';
    } else {
        $hdrs_type = 'HTTP/1.1';
    }

    if ($this->fault) {
        $hdrs = "$hdrs_type 500 Soap Fault\r\n";
        $response = $this->fault->message();
    } else {
        $hdrs = "$hdrs_type 200 OK\r\n";
    }

    header($hdrs);
    $this->headers['Server'] = SOAP_LIBRARY_NAME;
    if (!isset($this->headers['Content-Type'])) {
        $this->headers['Content-Type'] = 'text/xml; charset=' .
            $this->response_encoding;
    }
    $this->headers['Content-Length'] = strlen($response);

    foreach ($this->headers as $k => $v) {
        header("$k: $v");
        $hdrs .= "$k: $v\r\n";
    }
    header('SOAP-ProcessingTime: '.timeDiff(SOAP_PROCESSING_START, microtime(), false).' s');

    $this->response = $hdrs . "\r\n" . $response;
    print $response;
  } // _sendResponse()

  /**
    * overrides SOAP_Base_Object::_isHash()
    * 
    * sequences may have only 1 element, allow more than 10 elements either
    */
  function _isHash(&$a) {
      // I really dislike having to loop through this in PHP code, really
      // large arrays will be slow.  We need a C function to do this.
      $names = array();
      //!XXX - CS 
      // $it = 0; 
      foreach ($a as $k => $v) {
          // Checking the type is faster than regexp.
          $t = gettype($k);
          if ($t != 'integer') {
              return true;
          } elseif ($this->_isSoapValue($v)) {
              if ($v->name) { //!XXX + CS
                $names[$v->name] = 1;
              }
          }
          //!XXX - CS
          // If someone has a large hash they should really be defining the
          // type.
          // if ($it++ > 10) {
          //    return false;
          // }
      }
      return count($names)>0; //!XXX ~ CS
  } // _isHash()

 /**
    * overrides SOAP_Base_Object::_makeMimeMessage()
    *
    * dont encode message part
    */
  function &_makeMimeMessage(&$xml, $encoding = SOAP_DEFAULT_ENCODING) {
      global $SOAP_options;

      if (!isset($SOAP_options['Mime'])) {
          return $this->_raiseSoapFault('Mime is not installed');
      }

      // Encode any attachments.
      // See http://www.w3.org/TR/SOAP-attachments
      // Now we have to mime encode the message.
      $params = array('content_type' => 'multipart/related; type=text/xml');
      $msg =& new Mail_mimePart('', $params);

      // Add the xml part.
      $params['content_type'] = 'text/xml';
      $params['charset'] = $encoding;
      //!XXX - CS
      // $params['encoding'] = 'base64';
      $msg->addSubPart($xml, $params);

      // Add the attachements
      $c = count($this->__attachments);
      for ($i = 0; $i < $c; $i++) {
          $attachment =& $this->__attachments[$i];
          $msg->addSubPart($attachment['body'], $attachment);
      }

      return $msg->encode();
  } // _makeMimeMessage()

  function _decodeMimeMessage(&$data, &$headers, &$attachments) {
      global $SOAP_options;

      if (!isset($SOAP_options['Mime'])) {
          $this->_raiseSoapFault('Mime Unsupported, install PEAR::Mail::Mime', '', '', 'Server');
          return;
      }

      $params['include_bodies'] = true;
      $params['decode_bodies']  = true;
      $params['decode_headers'] = true;

      // Lame thing to have to do for decoding.
      $decoder =& new Mail_mimeDecode($data);
      $structure = $decoder->decode($params);

      if (isset($structure->body)) {
          $data = $structure->body;
          $headers = $structure->headers;
          return;
      } elseif (isset($structure->parts)) {
          $data = $structure->parts[0]->body;
          $headers = array_merge($structure->headers,
                                 $structure->parts[0]->headers);
          if (count($structure->parts) > 1) {
              $mime_parts = array_splice($structure->parts,1);
              // Prepare the parts for the SOAP parser.
              $c = count($mime_parts);
              for ($i = 0; $i < $c; $i++) {
                  $p =& $mime_parts[$i];
                  //!XXX +~ CS
                  $attachment =
                            array(
                                  new SOAP_Value('type', 'string',        array_get_value('content-type', $p->headers)),
                                  new SOAP_Value('location', 'string',    array_get_value('content-location', $p->headers)),
                                  new SOAP_Value('description', 'string', array_get_value('content-description', $p->headers)),
                                  new SOAP_Value('encoding', 'string',    array_get_value('content-transfer-encoding', $p->headers)),
                                  new SOAP_Value('size', 'string',        strlen($p->body)),
                                  new SOAP_Value('id', 'string',          array_get_value('content-id', $p->headers)),
                                  new SOAP_Value('name', 'string',        array_get_value('content-disposition', $p->headers)),
                                  new SOAP_Value('content', 'string',     $p->body));

                  $cid = 'cid:'.substr($p->headers['content-id'], 1, strlen($p->headers['content-id']) - 2);
                  $attachments[$cid] = $attachment; 
                  unset($p);
                  /*
                  if (isset($p->headers['content-location'])) {
                      // TODO: modify location per SwA note section 3
                      // http://www.w3.org/TR/SOAP-attachments
                      $attachments[$p->headers['content-location']] = $p->body;
                  } else {
                      $cid = 'cid:' . substr($p->headers['content-id'], 1, -2);
                      $attachments[$cid] = $p->body;
                  }
                  */
              }
          }

          return;
      }

      $this->_raiseSoapFault('Mime parsing error', '', '', 'Server');
  } // _decodeMimeMessage()
    
  /**
   * overrides SOAP_Server::service
   *
   * use Mime encoding as default
   */
  function service($data, $endpoint = '', $test = false) {
      $response = null;
      $attachments = array();
      $useEncoding = 'Mime'; //!XXX ~ CS
      
      
      /* Figure out our endpoint. */
      $this->endpoint = $endpoint;
      if (!$test && !$this->endpoint) {
          /* We'll try to build our endpoint. */
          $this->endpoint = 'http://' . $_SERVER['SERVER_NAME'];
          if ($_SERVER['SERVER_PORT']) {
              $this->endpoint .= ':' . $_SERVER['SERVER_PORT'];
          }
          $this->endpoint .= $_SERVER['SCRIPT_NAME'];
      }

      /* Get the character encoding of the incoming request treat incoming
       * data as UTF-8 if no encoding set. */
      if (isset($_SERVER['CONTENT_TYPE'])) {
          if (strcasecmp($_SERVER['CONTENT_TYPE'], 'application/dime') == 0) {
              $this->_decodeDIMEMessage($data, $headers, $attachments);
              $useEncoding = 'DIME';
          } elseif (stristr($_SERVER['CONTENT_TYPE'], 'multipart/related')) {
              /* This is a mime message, let's decode it. */
              $data = 'Content-Type: ' .
                  stripslashes($_SERVER['CONTENT_TYPE']) .
                  "\r\n\r\n" . $data;
              $this->_decodeMimeMessage($data, $headers, $attachments);
              $useEncoding = 'Mime';
          }
          if (!isset($headers['content-type'])) {
              $headers['content-type'] = stripslashes($_SERVER['CONTENT_TYPE']);
          }
          if (!$this->fault &&
              !$this->_getContentEncoding($headers['content-type'])) {
              $this->xml_encoding = SOAP_DEFAULT_ENCODING;
              /* Found encoding we don't understand; return a fault. */
              $this->_raiseSoapFault('Unsupported encoding, use one of ISO-8859-1, US-ASCII, UTF-8', '', '', 'Server');
          }
      }
      soap_server_log_request($this->debugOutput($data, $headers, true));

      /* If this is not a POST with Content-Type text/xml, try to return a
       * WSDL file. */
      if (!$this->fault && !$test &&
          ($_SERVER['REQUEST_METHOD'] != 'POST' ||
           (strncmp($headers['content-type'], 'text/xml', 8) != 0
           && strncmp($headers['content-type'], 'application/xml', 15) != 0))) {
          /* This is not possibly a valid SOAP request, try to return a WSDL
           * file. */
          $this->_raiseSoapFault('Invalid SOAP request, must be POST with content-type: text/xml, got: ' . (isset($headers['content-type']) ? $headers['content-type'] : 'Nothing!'), '', '', 'Server');
      }

      if (!$this->fault) {
          /* $response is a SOAP_Msg object. */
          $soap_msg = $this->parseRequest($data, $attachments);
          /* Handle Mime or DIME encoding. */
          /* TODO: DIME decoding should move to the transport, do it here
           * for now and for ease of getting it done. */
          if (count($this->__attachments)) {
              if ($useEncoding == 'Mime') {
                  $soap_msg = $this->_makeMimeMessage($soap_msg);
              } else {
                  // default is dime
                  $soap_msg = $this->_makeDIMEMessage($soap_msg);
                  $this->headers['Content-Type'] = 'application/dime';
              }
              if (PEAR::isError($soap_msg)) {
                  return $this->raiseSoapFault($soap_msg);
              }
          }

          if (is_array($soap_msg)) {
              $response = $soap_msg['body'];
              if (count($soap_msg['headers'])) {
                  $this->headers = $soap_msg['headers'];
              }
          } else {
              $response = $soap_msg;
          }
      }
      soap_server_log_response($this->debugOutput($response, $this->headers, true));
      $this->_sendResponse($response);
  } // service()
  
  function debugOutput($str, $headers, $truncateAttachments = false) {
  /*
  $r = 'soap message: ';
  foreach($headers AS $key => $value) {
  $r .= $key.':'.$value."\n";
  }
  $r .= $str;
  return $r;*/
    ($truncateAttachments && is_array($headers) &&
        (($ct = array_get_value('Content-Type', $headers)) || ($ct = array_get_value('content-type', $headers)))  &&
        ereg('boundary="([^"]*)"', $ct, $rr) && count($rr) > 1 &&
        $boundary = $rr[1])
    || $boundary = null;

    if ($boundary) {
      import('util.Text');
      $segs = explode($boundary, $str);
      $result = $segs[0].$boundary.$segs[1].$boundary.$segs[2];
      for($i = 3; $i < count($segs); ++$i) {
        $result .= $boundary.Text::abbrev($segs[$i], 500)."\n";
      }
      return $result;
    }
    return $str;
  } // debugOutput()


} // class SoapServer5
?>
