<!--
/******************************************************************************/
var Moodle = {};

/******************************************************************************/
/**
  * Moodle.API
  * Proxy Klasse fuer Moodle Soap Aufrufe
  *
  * Beispiel:
  * 
  * // Aufruf der Soap Operation 'foo' mit den Parametern
  * // p1 und p2  
  * // 
  * var response = (new Moodle.API()).foo(p1, p1);
  *
  */
Moodle.API = Class.create();
Moodle.API.prototype = {
  initialize :
    function(callback, endpoint) {
      if (callback) {
        Moodle.callback = callback;
      }
      if (!Moodle.callback) {
        Moodle.callback = new Moodle.DefaultCallback();
      }
      if (endpoint) {
        this.endpoint = endpoint;
      }
      else {
        this.endpoint = 'http://lms2.cms.hu-berlin.de/ws/soap/?Moodle';
      }
      this.fault     = null;
      this.result    = null;
    },
 
  /**
    * Liste von Moodle-Kursen
    */
  getCourses :
    function(
      /* moodlexsd:ArrayOfCondition[] */
      conditions) {
      var call   = new WS.Call(this.endpoint);
      var method = new WS.QName('getCourses', 'http://lms.hu-berlin.de/moodle');
      call.invoke_rpc(method, new Array(
                                        {name:'conditions', value:conditions}
                                        ),
                      null, // keine Angaben zu encoding ... rpc/literal ist wohl default...?
                      this.getCourses_deserialize);
    },
  getCourses_deserialize :
    function(wsCall, responseEnvelope, responseText) {
      this.result = null;
      do {
        if (!responseEnvelope) {
          this.result = new SOAP.Fault('SOAP:Client', 'Service returned an empty document');
          break;
        }
        var responseNode = SOAP.Helper.getResponseNode(responseEnvelope);
        if (!responseNode) break;
        if (SOAP.Helper.nodeNameEquals(responseNode, 'Fault')) {
          this.result = SOAP.Fault.fromNode(responseNode);
          break;
        }
        var resultNode = null;
        var resultNode = SOAP.Helper.getChildByName(responseNode, 'courses');
        if (!resultNode) break;
        this.result = Moodle.ArrayOfCourse.deserialize(resultNode);
        
      } while(false);
      if (Moodle.callback) {
        Moodle.callback.getCoursesResponse(this.result, wsCall, responseEnvelope, responseText);
      }
    },
  
  /**
    * Liste von Moodle Kategorien (HU Struktur hierarchisch)
    */
  getCategories :
    function(
      /* moodlexsd:ArrayOfCondition[] */
      conditions) {
      var call   = new WS.Call(this.endpoint);
      var method = new WS.QName('getCategories', 'http://lms.hu-berlin.de/moodle');
      call.invoke_rpc(method, new Array(
                                        {name:'conditions', value:conditions}
                                        ),
                      null, // keine Angaben zu encoding ... rpc/literal ist wohl default...?
                      this.getCategories_deserialize);
    },
  getCategories_deserialize :
    function(wsCall, responseEnvelope, responseText) {
      this.result = null;
      do {
        if (!responseEnvelope) {
          this.result = new SOAP.Fault('SOAP:Client', 'Service returned an empty document');
          break;
        }
        var responseNode = SOAP.Helper.getResponseNode(responseEnvelope);
        if (!responseNode) break;
        if (SOAP.Helper.nodeNameEquals(responseNode, 'Fault')) {
          this.result = SOAP.Fault.fromNode(responseNode);
          break;
        }
        var resultNode = null;
        var resultNode = SOAP.Helper.getChildByName(responseNode, 'categories');
        if (!resultNode) break;
        this.result = Moodle.ArrayOfCategory.deserialize(resultNode);
        
      } while(false);
      if (Moodle.callback) {
        Moodle.callback.getCategoriesResponse(this.result, wsCall, responseEnvelope, responseText);
      }
    }
  
}

Moodle.DefaultCallback = Class.create();
Moodle.DefaultCallback.prototype = {
  initialize :
    function() {
    },
 
  /**
    * Liste von Moodle-Kursen
    */
  getCoursesResponse :
    function(result, wsCall, responseEnvelope, responseText) {
      if (SOAP.DEBUG) {
        alert('Moodle.DefaultCallback:getCoursesResponse ' + result);
      }
    },
  
  /**
    * Liste von Moodle Kategorien (HU Struktur hierarchisch)
    */
  getCategoriesResponse :
    function(result, wsCall, responseEnvelope, responseText) {
      if (SOAP.DEBUG) {
        alert('Moodle.DefaultCallback:getCategoriesResponse ' + result);
      }
    }
  
};

//
/******************************************************************************/
//
Moodle.String = Class.create();
Moodle.String.prototype = (new SOAP.Object()).extend({
  initialize :
    function() {
      if (arguments[0]) {
        this.value = arguments[0];
      }
      else {
        this.value = null;
      }
    }
});
Moodle.String.deserialize =
    function(element) {
      if (element && element.firstChild) {
        return element.firstChild.data;
      }
      return '';
    };
//
/******************************************************************************/
//
Moodle.Condition = Class.create();
Moodle.Condition.prototype = (new SOAP.Sequence()).extend({
  initialize :
    function(/* string */ expr,
             /* string */ op,
             /* string */ value
             ) {
      this.values = new Array();
      if (arguments && arguments.length) {
      this.values['expr'] = expr;
      this.values['op'] = op;
      this.values['value'] = value;
      }
    },
  getExpr :
    function() {
      return this.values['expr'];
    },
  setExpr :
    function(v) {
      this.values['expr'] = v;
    },
  getOp :
    function() {
      return this.values['op'];
    },
  setOp :
    function(v) {
      this.values['op'] = v;
    },
  getValue :
    function() {
      return this.values['value'];
    },
  setValue :
    function(v) {
      this.values['value'] = v;
    },
  serialize :
    function(element) {
      if (!(this.values.expr || this.values.expr instanceof SOAP.Object || this.values.op || this.values.op instanceof SOAP.Object || this.values.value || this.values.value instanceof SOAP.Object || 0)) {
        element.set_attribute(SOAP.XSINIL, 'true');
        return;
      }
      
      this.do_serialize(element.create_child(new WS.QName('expr', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.expr, 'string', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('op', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.op, 'string', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('value', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.value, 'string', 'http://www.w3.org/2001/XMLSchema', 'xsd');
    }
});

Moodle.Condition.deserialize =
    function(element) {
      var res = new Moodle.Condition();
      var node;
      if ((node = SOAP.Helper.getChildByName(element, 'expr')) != null)
        res.values['expr'] = Moodle.String.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'op')) != null)
        res.values['op'] = Moodle.String.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'value')) != null)
        res.values['value'] = Moodle.String.deserialize(node);
      return res;
    };

//
/******************************************************************************/
//
Moodle.ArrayOfCondition = Class.create();
Moodle.ArrayOfCondition.prototype = (new SOAP.Array()).extend({
  initialize :
    function() {
      if (arguments[0]) {
        this.values = arguments[0];
      }
      else {
        this.values = new Array();
      }
    },
  serialize :
    function(element) {
      this.serialize_array(element, 'http://lms.hu-berlin.de/moodle', 'moodle', 'Condition', 'http://lms.hu-berlin.de/moodle/Schema', 'moodlexsd');
    }
});
Moodle.ArrayOfCondition.deserialize =
    function(element) {
      var children = SOAP.Helper.getChildren(element);
      var values = new Array();
      for(var i = 0; i < children.length; ++i) {
        values.push(Moodle.Condition.deserialize(children[i]));
      }
      return values;
    };
//
/******************************************************************************/
//
Moodle.Int = Class.create();
Moodle.Int.prototype = (new SOAP.Object()).extend({
  initialize :
    function() {
      if (arguments[0]) {
        this.value = arguments[0];
      }
      else {
        this.value = null;
      }
    }
});
Moodle.Int.deserialize =
    function(element) {
      if (element && element.firstChild) {
        return element.firstChild.data;
      }
      return '';
    };
//
/******************************************************************************/
//
Moodle.Boolean = Class.create();
Moodle.Boolean.prototype = (new SOAP.Object()).extend({
  initialize :
    function() {
      if (arguments[0]) {
        this.value = arguments[0];
      }
      else {
        this.value = null;
      }
    }
});
Moodle.Boolean.deserialize =
    function(element) {
      if (element && element.firstChild) {
        return element.firstChild.data;
      }
      return '';
    };
//
/******************************************************************************/
//
Moodle.Course = Class.create();
Moodle.Course.prototype = (new SOAP.Sequence()).extend({
  initialize :
    function(/* int */ id,
             /* string */ title,
             /* string */ shortcut,
             /* string */ summary,
             /* boolean */ public,
             /* boolean */ allowGuest,
             /* string */ location,
             /* string */ teachers
             ) {
      this.values = new Array();
      if (arguments && arguments.length) {
      this.values['id'] = id;
      this.values['title'] = title;
      this.values['shortcut'] = shortcut;
      this.values['summary'] = summary;
      this.values['public'] = public;
      this.values['allowGuest'] = allowGuest;
      this.values['location'] = location;
      this.values['teachers'] = teachers;
      }
    },
  getId :
    function() {
      return this.values['id'];
    },
  setId :
    function(v) {
      this.values['id'] = v;
    },
  getTitle :
    function() {
      return this.values['title'];
    },
  setTitle :
    function(v) {
      this.values['title'] = v;
    },
  getShortcut :
    function() {
      return this.values['shortcut'];
    },
  setShortcut :
    function(v) {
      this.values['shortcut'] = v;
    },
  getSummary :
    function() {
      return this.values['summary'];
    },
  setSummary :
    function(v) {
      this.values['summary'] = v;
    },
  getPublic :
    function() {
      return this.values['public'];
    },
  setPublic :
    function(v) {
      this.values['public'] = v;
    },
  getAllowGuest :
    function() {
      return this.values['allowGuest'];
    },
  setAllowGuest :
    function(v) {
      this.values['allowGuest'] = v;
    },
  getLocation :
    function() {
      return this.values['location'];
    },
  setLocation :
    function(v) {
      this.values['location'] = v;
    },
  getTeachers :
    function() {
      return this.values['teachers'];
    },
  setTeachers :
    function(v) {
      this.values['teachers'] = v;
    },
  serialize :
    function(element) {
      if (!(this.values.id || this.values.id instanceof SOAP.Object || this.values.title || this.values.title instanceof SOAP.Object || this.values.shortcut || this.values.shortcut instanceof SOAP.Object || this.values.summary || this.values.summary instanceof SOAP.Object || this.values.public || this.values.public instanceof SOAP.Object || this.values.allowGuest || this.values.allowGuest instanceof SOAP.Object || this.values.location || this.values.location instanceof SOAP.Object || this.values.teachers || this.values.teachers instanceof SOAP.Object || 0)) {
        element.set_attribute(SOAP.XSINIL, 'true');
        return;
      }
      
      this.do_serialize(element.create_child(new WS.QName('id', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.id, 'int', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('title', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.title, 'string', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('shortcut', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.shortcut, 'string', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('summary', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.summary, 'string', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('public', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.public, 'boolean', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('allowGuest', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.allowGuest, 'boolean', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('location', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.location, 'string', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('teachers', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.teachers, 'string', 'http://www.w3.org/2001/XMLSchema', 'xsd');
    }
});

Moodle.Course.deserialize =
    function(element) {
      var res = new Moodle.Course();
      var node;
      if ((node = SOAP.Helper.getChildByName(element, 'id')) != null)
        res.values['id'] = Moodle.Int.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'title')) != null)
        res.values['title'] = Moodle.String.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'shortcut')) != null)
        res.values['shortcut'] = Moodle.String.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'summary')) != null)
        res.values['summary'] = Moodle.String.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'public')) != null)
        res.values['public'] = Moodle.Boolean.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'allowGuest')) != null)
        res.values['allowGuest'] = Moodle.Boolean.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'location')) != null)
        res.values['location'] = Moodle.String.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'teachers')) != null)
        res.values['teachers'] = Moodle.String.deserialize(node);
      return res;
    };

//
/******************************************************************************/
//
Moodle.ArrayOfCourse = Class.create();
Moodle.ArrayOfCourse.prototype = (new SOAP.Array()).extend({
  initialize :
    function() {
      if (arguments[0]) {
        this.values = arguments[0];
      }
      else {
        this.values = new Array();
      }
    },
  serialize :
    function(element) {
      this.serialize_array(element, 'http://lms.hu-berlin.de/moodle', 'moodle', 'Course', 'http://lms.hu-berlin.de/moodle/Schema', 'moodlexsd');
    }
});
Moodle.ArrayOfCourse.deserialize =
    function(element) {
      var children = SOAP.Helper.getChildren(element);
      var values = new Array();
      for(var i = 0; i < children.length; ++i) {
        values.push(Moodle.Course.deserialize(children[i]));
      }
      return values;
    };
//
/******************************************************************************/
//
Moodle.Category = Class.create();
Moodle.Category.prototype = (new SOAP.Sequence()).extend({
  initialize :
    function(/* int */ id,
             /* string */ title,
             /* string */ location
             ) {
      this.values = new Array();
      if (arguments && arguments.length) {
      this.values['id'] = id;
      this.values['title'] = title;
      this.values['location'] = location;
      }
    },
  getId :
    function() {
      return this.values['id'];
    },
  setId :
    function(v) {
      this.values['id'] = v;
    },
  getTitle :
    function() {
      return this.values['title'];
    },
  setTitle :
    function(v) {
      this.values['title'] = v;
    },
  getLocation :
    function() {
      return this.values['location'];
    },
  setLocation :
    function(v) {
      this.values['location'] = v;
    },
  serialize :
    function(element) {
      if (!(this.values.id || this.values.id instanceof SOAP.Object || this.values.title || this.values.title instanceof SOAP.Object || this.values.location || this.values.location instanceof SOAP.Object || 0)) {
        element.set_attribute(SOAP.XSINIL, 'true');
        return;
      }
      
      this.do_serialize(element.create_child(new WS.QName('id', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.id, 'int', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('title', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.title, 'string', 'http://www.w3.org/2001/XMLSchema', 'xsd');
      this.do_serialize(element.create_child(new WS.QName('location', 'http://lms.hu-berlin.de/moodle', 'moodle')), this.values.location, 'string', 'http://www.w3.org/2001/XMLSchema', 'xsd');
    }
});

Moodle.Category.deserialize =
    function(element) {
      var res = new Moodle.Category();
      var node;
      if ((node = SOAP.Helper.getChildByName(element, 'id')) != null)
        res.values['id'] = Moodle.Int.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'title')) != null)
        res.values['title'] = Moodle.String.deserialize(node);
      if ((node = SOAP.Helper.getChildByName(element, 'location')) != null)
        res.values['location'] = Moodle.String.deserialize(node);
      return res;
    };

//
/******************************************************************************/
//
Moodle.ArrayOfCategory = Class.create();
Moodle.ArrayOfCategory.prototype = (new SOAP.Array()).extend({
  initialize :
    function() {
      if (arguments[0]) {
        this.values = arguments[0];
      }
      else {
        this.values = new Array();
      }
    },
  serialize :
    function(element) {
      this.serialize_array(element, 'http://lms.hu-berlin.de/moodle', 'moodle', 'Category', 'http://lms.hu-berlin.de/moodle/Schema', 'moodlexsd');
    }
});
Moodle.ArrayOfCategory.deserialize =
    function(element) {
      var children = SOAP.Helper.getChildren(element);
      var values = new Array();
      for(var i = 0; i < children.length; ++i) {
        values.push(Moodle.Category.deserialize(children[i]));
      }
      return values;
    };

// -->
